package com.itheima.screen;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.DisplayMode;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.text.DateFormat;
import java.util.Date;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * 这是一个LCD样式的时钟屏幕保护程序 
 * 为了能够显示LCD数字的效果，请安装我在压缩包里面提供的字体
 * 
 * @author mrng
 */
@SuppressWarnings("serial")
public class LCDClock extends JPanel {
	// 主函数
	public static void main(String arsg[]) {
		/****** 如果想要使时钟移动，就在其构造器里面传入true ***********/

		// 创建LCDClock实例，false表示时钟不在屏幕上移动
		// final LCDClock clock = new LCDClock(false);

		// 创建LCDClock实例，true表示时钟在屏幕上移动
		final LCDClock clock = new LCDClock(true);

		// 启动clock
		clock.open();

	}

	// 启动时钟
	public void open() {
		Executors.newScheduledThreadPool(1).scheduleAtFixedRate(new Runnable() {
			public void run() {
				panel.updateTime();
				// 每过3毫秒重绘一次组件
				panel.repaint();
			} // 这里是3毫秒循环一次
		}, 0, 3, TimeUnit.MILLISECONDS);
	}

	/*-------------------------------------------------------------------------------------*/

	// 顶层窗口
	JFrame frame;
	// 显示时钟的面板
	Panel panel;
	// 用于指定时钟是否漂移
	private boolean fly;

	public LCDClock(boolean fly) {
		this.fly = fly;
		// 初始化frame
		initFrame();
	}

	// 对frame进行初始化
	private void initFrame() {
		// 顶层窗口
		frame = new JFrame();

		// 得到描述当前图形设备(屏幕)的GraphicsDevice实例
		GraphicsDevice graphicsDevice = GraphicsEnvironment
				.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		// 获得当前GraphicsDevice的显示模式
		DisplayMode displayMode = graphicsDevice.getDisplayMode();

		// 获得当前图形设备的宽度，单位为像素
		int width = displayMode.getWidth();
		// 获得当前图形设备的高度，单位为像素
		int height = displayMode.getHeight();

		// 创建显示时钟的面板
		panel = new Panel(width, height);

		// 得到Container容器，并将面板添加进去，并至于中间位置(填充整个屏幕)
		frame.getContentPane().add(panel, BorderLayout.CENTER);
		// 禁用窗体的装饰
		frame.setUndecorated(true);
		// 设置为全屏显示
		graphicsDevice.setFullScreenWindow(frame);

		// 添加退出事件
		initEvent(frame);
	}

	// 初始化退出事件
	public void initEvent(JFrame jFrame) {
		// 添加键盘事件
		jFrame.addKeyListener(new KeyAdapter() {
			@SuppressWarnings("static-access")
			public void keyReleased(KeyEvent e) {
				// 按下ENTER或ESC键退出
				if (e.getKeyCode() == e.VK_ENTER
						|| e.getKeyCode() == e.VK_ESCAPE)
					System.exit(0);
			}
		});

		// 添加鼠标事件
		jFrame.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				// 双击鼠标退出
				if (e.getClickCount() == 2) {// 如果双击鼠标，程序退出
					System.exit(0);
				}
			}
		});
	}

	/*-------------------------------------------------------------------------------------*/

	// 显示时钟的面板
	private class Panel extends JPanel {
		// 屏幕的尺寸
		private int width, height;
		// 字体的颜色
		private Color fontColor;
		// 表示当前时间的字符串
		private String currentTime;
		// 代表当前图形中显示的字符串的边界框，也就是时间字符串的边界框
		private Rectangle2D rectangle2d;
		// x是横坐标，y是纵座标，lr表示向左或又移动，tb表示向上或下移动
		private int x, y, lr, tb;

		// 由创建者指定尺寸
		public Panel(int width, int height) {
			this.width = width;
			this.height = height;
			// 初始化面板
			init();
		}

		// 初始化面板
		private void init() {
			// 得到时钟的起始位置，屏幕的中央
			x = (width / 2) - (878 / 2);
			y = (height / 2) - (219 / 2);
			lr = 1;
			tb = 1;

			rectangle2d = new Rectangle2D.Double();
			fontColor = new Color(224, 218, 252);
		}

		// 绘制面板中的每个组件
		public void paintComponent(Graphics g) {
			// 设置背景色
			g.setColor(new Color(20, 30, 50));
			// 按指定的尺寸填充面板
			g.fillRect(0, 0, width, height);

			// 设置字体样式
			g.setFont(new Font("DigifaceWide", Font.BOLD, 180));
			// 设置字体的颜色
			g.setColor(fontColor);

			// 获取当前字体的规格
			FontMetrics fontMetrics = g.getFontMetrics();
			// 得到代表当前图形中显示的字符串的边界框，也就是时间字符串的边界框，用来对时间字符串进行布局
			rectangle2d = fontMetrics.getStringBounds(currentTime, g);

			// 设置时钟的坐标和大小
			rectangle2d.setRect(x, y, rectangle2d.getWidth(),
					rectangle2d.getHeight());
			// 将时间绘制到面板上
			g.drawString(currentTime, x, (int) (y + rectangle2d.getHeight()));

			// 重新设置字体大小
			g.setFont(new Font("DigifaceWide", Font.BOLD, 50));
			// 添加新的字样
			g.drawString("CSDN 中关村软件园 传智播客 强强联手", x, 50);
			g.drawString(" 黑马程序员-www.itheima.com", x, 110);
			// 将时间绘制到面板上
			// g.drawString("黑马时间", 10, (int) rectangle2d2.getHeight() - 165);
			// g.drawString("黑马程序员-www.itheima.com", 10, height - 20);

		}

		// 改变字体的颜色
		private void changeColor() {
			// 变换颜色时从这里随机获取一个
			Color[] colors = { new Color(224, 218, 252),
					new Color(255, 131, 132), new Color(199, 216, 112),
					new Color(75, 140, 248), new Color(137, 182, 115),
					new Color(199, 120, 59), Color.ORANGE };

			// 得到一个随机颜色并改变字体颜色
			Color newColor = null;
			while (true) {
				newColor = colors[(int) (Math.random() * 7)];
				// 如果获取到的随机颜色和当前的相同，则重新获取
				if (newColor.equals(fontColor))
					continue;
				else
					break;
			}
			fontColor = newColor;
		}

		public void updateTime() {
			// 得到日期格式化对象
			DateFormat format = DateFormat.getTimeInstance(DateFormat.MEDIUM);
			// 创建代表当前时间的date
			Date date = new Date();
			// 将当前时间按时分秒格式化并得到其对应的字符串
			currentTime = format.format(date);

			// 如果不漂移，就不动态的改变坐标, 在屏幕中间位置显示
			if (!fly) {
				return;
			} else {
				// 移动
				move();
			}
		}

		// 移动时钟
		public void move() {
			// 动态的改变坐标
			x += lr;
			y += tb;

			// 在测试的时候时钟移动时发虚很厉害，后来发现是这句话惹的祸，呵呵......
			// System.out.println(rectangle2d.getWidth() + "................." +
			// rectangle2d.getHeight());

			// 判断时钟是否已经移动到了屏幕的边缘
			// 如果此时时钟已到达屏幕的上边缘
			// 则重新上色，并向下边移动
			if (rectangle2d.intersectsLine(-90, -90, width, -90)) {
				changeColor();
				tb = 1;

			} else
			// 如果此时Rectangle2D已到达屏幕的下边缘
			// 则重新上色，并向上边移动
			if (rectangle2d.intersectsLine(0, height, width, height)) {
				changeColor();
				tb = -1;
			} else
			// 如果此时Rectangle2D已到达屏幕的左边缘
			// 则重新上色，并向右边移动
			if (rectangle2d.intersectsLine(0, 0, 0, height)) {
				changeColor();
				lr = 1;
			} else
			// 如果此时时钟已到达屏幕的右边缘
			// 则重新上色，并向左边移动
			if (rectangle2d.intersectsLine(width, 0, width, height)) {
				changeColor();
				lr = -1;
			}
		}
	}

}